@@ -18,6 +18,7 @@ import iso8601
 import mock
 import netaddr
 import six
+import testtools
 
 from oslo_versionedobjects import _utils
 from oslo_versionedobjects import base as obj_base
@@ -65,6 +66,32 @@ class FakeEnumField(fields.BaseEnumField):
     AUTO_TYPE = FakeEnum()
 
 
+class FakeStateMachineField(fields.StateMachine):
+
+    ACTIVE = 'ACTIVE'
+    PENDING = 'PENDING'
+    ERROR = 'ERROR'
+
+    ALLOWED_TRANSITIONS = {
+        ACTIVE: {
+            PENDING,
+            ERROR
+        },
+        PENDING: {
+            ACTIVE,
+            ERROR
+        },
+        ERROR: {
+            PENDING,
+        },
+    }
+
+    _TYPES = (ACTIVE, PENDING, ERROR)
+
+    def __init__(self, **kwargs):
+        super(FakeStateMachineField, self).__init__(self._TYPES, **kwargs)
+
+
 class FakeEnumAltField(fields.BaseEnumField):
     AUTO_TYPE = FakeEnumAlt()
 
@@ -164,6 +191,43 @@ class TestMACAddress(TestField):
         self.from_primitive_values = self.coerce_good_values[0:1]
 
 
+class TestUUID(TestField):
+    def setUp(self):
+        super(TestUUID, self).setUp()
+
+    def test_validation_enabled(self):
+        self.field = fields.UUIDField()
+        self.coerce_good_values = [
+            ('da66a411-af0e-4829-9b67-475017ddd152',
+                'da66a411-af0e-4829-9b67-475017ddd152'),
+            ('da66a411af0e48299b67475017ddd152',
+                'da66a411af0e48299b67475017ddd152'),
+            ('DA66A411-AF0E-4829-9B67-475017DDD152',
+                'DA66A411-AF0E-4829-9B67-475017DDD152'),
+            ('DA66A411AF0E48299b67475017DDD152',
+                'DA66A411AF0E48299b67475017DDD152'),
+            # These values are included to ensure there is not change in
+            # behaviour - only when we can remove the old UUID behaviour can
+            #  we add these to the "self.coerce_bad_values" list
+            ('da66a411-af0e-4829-9b67',
+                'da66a411-af0e-4829-9b67'),
+            ('da66a411-af0e-4829-9b67-475017ddd152548999',
+                'da66a411-af0e-4829-9b67-475017ddd152548999'),
+            ('da66a411-af0e-4829-9b67-475017ddz152',
+                'da66a411-af0e-4829-9b67-475017ddz152'),
+            ('fake_uuid', 'fake_uuid'),
+            ('1', '1'),
+            (1, '1')
+        ]
+
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+        self.test_coerce_good_values()
+        self.test_from_primitive()
+        self.test_to_primitive()
+
+
 class TestBaseEnum(TestField):
     def setUp(self):
         super(TestBaseEnum, self).setUp()
@@ -225,6 +289,66 @@ class TestEnum(TestField):
                           fields.EnumField, True)
 
 
+class TestStateMachine(TestField):
+
+    def test_good_transitions(self):
+        @obj_base.VersionedObjectRegistry.register
+        class AnObject(obj_base.VersionedObject):
+            fields = {
+                'status': FakeStateMachineField(),
+            }
+
+        obj = AnObject()
+
+        obj.status = FakeStateMachineField.ACTIVE
+        obj.status = FakeStateMachineField.PENDING
+        obj.status = FakeStateMachineField.ERROR
+        obj.status = FakeStateMachineField.PENDING
+        obj.status = FakeStateMachineField.ACTIVE
+
+    def test_bad_transitions(self):
+        @obj_base.VersionedObjectRegistry.register
+        class AnObject(obj_base.VersionedObject):
+            fields = {
+                'status': FakeStateMachineField(),
+            }
+
+        obj = AnObject()
+
+        with testtools.ExpectedException(
+                ValueError,
+                msg="AnObject's are not allowed transition out "
+                "of 'ERROR' state to 'PENDING' state, choose from "
+                "['PENDING']"):
+            obj.status = FakeStateMachineField.ERROR
+            obj.status = FakeStateMachineField.ACTIVE
+
+    def test_bad_initial_value(self):
+        @obj_base.VersionedObjectRegistry.register
+        class AnObject(obj_base.VersionedObject):
+            fields = {
+                'status': FakeStateMachineField(),
+            }
+
+        obj = AnObject()
+
+        with testtools.ExpectedException(ValueError):
+            obj.status = "FOO"
+
+    def test_bad_updated_value(self):
+        @obj_base.VersionedObjectRegistry.register
+        class AnObject(obj_base.VersionedObject):
+            fields = {
+                'status': FakeStateMachineField(),
+            }
+
+        obj = AnObject()
+
+        with testtools.ExpectedException(ValueError):
+            obj.status = FakeStateMachineField.ACTIVE
+            obj.status = "FOO"
+
+
 class TestInteger(TestField):
     def setUp(self):
         super(TestField, self).setUp()
